@@ -1,7 +1,7 @@
 #copyright ReportLab Inc. 2000-2001
 #see license.txt for license details
 #history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/graphics/charts/axes.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/graphics/charts/axes.py,v 1.61 2002/12/02 19:38:57 rgbecker Exp $
+#$Header: /cvsroot/reportlab/reportlab/graphics/charts/axes.py,v 1.80 2003/07/02 16:10:10 rgbecker Exp $
 """Collection of axes for charts.
 
 The current collection comprises axes for charts using cartesian
@@ -31,7 +31,7 @@ connection can be either at the top or bottom of the former or
 at any absolute value (specified in points) or at some value of
 the former axes in its own coordinate system.
 """
-__version__=''' $Id: axes.py,v 1.61 2002/12/02 19:38:57 rgbecker Exp $ '''
+__version__=''' $Id: axes.py,v 1.80 2003/07/02 16:10:10 rgbecker Exp $ '''
 
 import string
 from types import FunctionType, StringType, TupleType, ListType
@@ -73,6 +73,7 @@ class CategoryAxis(Widget):
         visible = AttrMapValue(isBoolean, desc='Display entire object, if true.'),
         visibleAxis = AttrMapValue(isBoolean, desc='Display axis line, if true.'),
         visibleTicks = AttrMapValue(isBoolean, desc='Display axis ticks, if true.'),
+        visibleLabels = AttrMapValue(isBoolean, desc='Display axis labels, if true.'),
         visibleGrid = AttrMapValue(isBoolean, desc='Display axis grid, if true.'),
         strokeWidth = AttrMapValue(isNumber, desc='Width of axis line and ticks.'),
         strokeColor = AttrMapValue(isColorOrNone, desc='Color of axis line and ticks.'),
@@ -80,8 +81,8 @@ class CategoryAxis(Widget):
         gridStrokeWidth = AttrMapValue(isNumber, desc='Width of grid lines.'),
         gridStrokeColor = AttrMapValue(isColorOrNone, desc='Color of grid lines.'),
         gridStrokeDashArray = AttrMapValue(isListOfNumbersOrNone, desc='Dash array used for grid lines.'),
-        gridStart = AttrMapValue(isNumber, desc='Start of grid lines wrt axis origin'),
-        gridEnd = AttrMapValue(isNumber, desc='End of grid lines wrt axis origin'),
+        gridStart = AttrMapValue(isNumberOrNone, desc='Start of grid lines wrt axis origin'),
+        gridEnd = AttrMapValue(isNumberOrNone, desc='End of grid lines wrt axis origin'),
         labels = AttrMapValue(None, desc='Handle of the axis labels.'),
         categoryNames = AttrMapValue(isListOfStringsOrNone, desc='List of category names.'),
         joinAxis = AttrMapValue(None, desc='Join both axes if true.'),
@@ -105,6 +106,7 @@ class CategoryAxis(Widget):
         self.visible = 1
         self.visibleAxis = 1
         self.visibleTicks = 1
+        self.visibleLabels = 1
         self.visibleGrid = 0
 
         self.strokeWidth = 1
@@ -126,6 +128,9 @@ class CategoryAxis(Widget):
         self.reverseDirection = 0
         self.style = 'parallel'
 
+        #various private things which need to be initialized
+        self._labelTextFormat = None
+
     def setPosition(self, x, y, length):
         # ensure floating point
         self._x = x
@@ -153,10 +158,18 @@ class CategoryAxis(Widget):
         if self.reverseDirection: idx = self._catCount-idx-1
         return idx
 
-    def makeGrid(self,g):
+    def makeGrid(self,g,dim=None):
         '''this is only called by a container object'''
-        if self.visibleGrid and (self.gridStart or self.gridEnd):
-            self._makeLines(g,self.gridStart,self.gridEnd,self.gridStrokeColor,self.gridStrokeWidth,self.gridStrokeDashArray)
+        s = self.gridStart
+        e = self.gridEnd
+        if dim:
+            s = s is None and dim[0]
+            e = e is None and dim[0]+dim[1]
+        c = self.gridStrokeColor
+        if self.visibleGrid and (s or e) and c is not None:
+            if self._dataIndex: offs = self._x
+            else: offs = self._y
+            self._makeLines(g,s-offs,e-offs,c,self.gridStrokeWidth,self.gridStrokeDashArray)
 
 def _assertYAxis(axis):
     acn = axis.__class__.__name__
@@ -177,6 +190,8 @@ class XCategoryAxis(CategoryAxis):
             desc="Mode used for connecting axis ('bottom', 'top', 'value', 'points', None)."),
         )
 
+    _dataIndex = 0
+
     def __init__(self):
         CategoryAxis.__init__(self)
         self.labels.boxAnchor = 'n' #north - top edge
@@ -279,14 +294,14 @@ class XCategoryAxis(CategoryAxis):
     def makeTickLabels(self):
         g = Group()
 
-        if not self.visibleTicks:
+        if not self.visibleLabels:
             return g
 
         if not (self.categoryNames is None):
             catCount = self._catCount
             assert len(self.categoryNames) == catCount, \
-                "expected %d category names but found %d in axis" % (
-                len(self.categoryNames), catCount
+                "expected %d category names but found %d in axis. \n categories = %s" % (
+                len(self.categoryNames), catCount, repr(self.categoryNames)
                 )
             reverseDirection = self.reverseDirection
             barWidth = self._barWidth
@@ -316,6 +331,8 @@ class YCategoryAxis(CategoryAxis):
             desc="Mode used for connecting axis ('left', 'right', 'value', 'points', None)."),
         )
 
+    _dataIndex = 1
+
 
     def __init__(self):
         CategoryAxis.__init__(self)
@@ -394,9 +411,9 @@ class YCategoryAxis(CategoryAxis):
         for i in range(self._catCount + 1):
             y = self._y + i*self._barWidth
             L = Line(self._x+start, y, self._x+end, y)
-            L.strokeColor = self.strokeColor
-            L.strokeWidth = self.strokeWidth
-            L.strokeDashArray = self.strokeDashArray
+            L.strokeColor = strokeColor
+            L.strokeWidth = strokeWidth
+            L.strokeDashArray = strokeDashArray
             g.add(L)
 
     def makeTicks(self):
@@ -449,7 +466,7 @@ class ValueAxis(Widget):
     "Abstract value axis, unusable in itself."
 
     _attrMap = AttrMap(
-        forceZero = AttrMapValue(isBoolean, desc='Ensure zero in range if true.'),
+        forceZero = AttrMapValue(EitherOr((isBoolean,OneOf('near'))), desc='Ensure zero in range if true.'),
         visible = AttrMapValue(isBoolean, desc='Display entire object, if true.'),
         visibleAxis = AttrMapValue(isBoolean, desc='Display axis line, if true.'),
         visibleTicks = AttrMapValue(isBoolean, desc='Display axis ticks, if true.'),
@@ -460,8 +477,8 @@ class ValueAxis(Widget):
         gridStrokeWidth = AttrMapValue(isNumber, desc='Width of grid lines.'),
         gridStrokeColor = AttrMapValue(isColorOrNone, desc='Color of grid lines.'),
         gridStrokeDashArray = AttrMapValue(isListOfNumbersOrNone, desc='Dash array used for grid lines.'),
-        gridStart = AttrMapValue(isNumber, desc='Start of grid lines wrt axis origin'),
-        gridEnd = AttrMapValue(isNumber, desc='End of grid lines wrt axis origin'),
+        gridStart = AttrMapValue(isNumberOrNone, desc='Start of grid lines wrt axis origin'),
+        gridEnd = AttrMapValue(isNumberOrNone, desc='End of grid lines wrt axis origin'),
         minimumTickSpacing = AttrMapValue(isNumber, desc='Minimum value for distance between ticks.'),
         maximumTicks = AttrMapValue(isNumber, desc='Maximum number of ticks.'),
         labels = AttrMapValue(None, desc='Handle of the axis labels.'),
@@ -470,8 +487,9 @@ class ValueAxis(Widget):
         valueMax = AttrMapValue(isNumberOrNone, desc='Maximum value on axis.'),
         valueStep = AttrMapValue(isNumberOrNone, desc='Step size used between ticks.'),
         valueSteps = AttrMapValue(isListOfNumbersOrNone, desc='List of step sizes used between ticks.'),
-        avoidBoundFrac = AttrMapValue(isNumberOrNone, desc='Fraction of interval to allow above and below.'),
+        avoidBoundFrac = AttrMapValue(EitherOr((isNumberOrNone,SequenceOf(isNumber,emptyOK=0,lo=2,hi=2))), desc='Fraction of interval to allow above and below.'),
         rangeRound=AttrMapValue(OneOf('none','both','ceiling','floor'),'How to round the axis limits'),
+        zrangePref = AttrMapValue(isNumberOrNone, desc='Zero range axis limit preference.'),
         )
 
     def __init__(self):
@@ -506,12 +524,9 @@ class ValueAxis(Widget):
         self.minimumTickSpacing = 10
         self.maximumTicks = 7
 
-        # this may be either of (a) a format string like '%0.2f'
-        # or (b) a function which takes the value as an argument
-        # and returns a chunk of text.  So you can write a
-        # 'formatMonthEndDate' function and use that on irregular
-        # data points.
-        self.labelTextFormat = '%d'
+        # a format string like '%0.2f'
+        # or a function which takes the value as an argument and returns a string
+        self._labelTextFormat = self.labelTextFormat = None
 
         # if set to None, these will be worked out for you.
         # if you override any or all of them, your values
@@ -521,7 +536,7 @@ class ValueAxis(Widget):
         self.valueStep = None
         self.avoidBoundFrac = None
         self.rangeRound = 'none'
-
+        self.zrangePref = 0
 
     def setPosition(self, x, y, length):
         # ensure floating point
@@ -529,7 +544,6 @@ class ValueAxis(Widget):
         self._y = y * 1.0
         self._length = length * 1.0
 
-
     def configure(self, dataSeries):
         """Let the axis configure its scale and range based on the data.
 
@@ -544,10 +558,31 @@ class ValueAxis(Widget):
         to use in plotting.
         """
         self._setRange(dataSeries)
-        self._calcScaleFactor()
         self._calcTickmarkPositions()
+        self._calcScaleFactor()
         self._configured = 1
 
+    def _getValueStepAndTicks(self, valueMin, valueMax,cache={}):
+        try:
+            K = (valueMin,valueMax)
+            r = cache[K]
+        except:
+            self._valueMin = valueMin
+            self._valueMax = valueMax
+            T = self._calcTickPositions()
+            if len(T)>1:
+                valueStep = T[1]-T[0]
+            else:
+                oVS = self.valueStep
+                self.valueStep = None
+                T = self._calcTickPositions()
+                self.valueStep = oVS
+                if len(T)>1:
+                    valueStep = T[1]-T[0]
+                else:
+                    valueStep = self._valueStep
+            r = cache[K] = valueStep, T, valueStep*1e-8
+        return r
 
     def _setRange(self, dataSeries):
         """Set minimum and maximum axis values.
@@ -563,45 +598,75 @@ class ValueAxis(Widget):
         if valueMax is None: self._cValueMax = valueMax = _findMax(dataSeries,self._dataIndex,0)
         if valueMin == valueMax:
             if valueMax==0:
-                valueMax = 0.01
-                valueMin = -0.01
-            else:
-                a = abs(valueMax)
-                valueMax = valueMax + a
-                valueMin = valueMin - a
-        if self.forceZero:
-            if valueMax<0: valueMax=0
-            elif valueMin>0: valueMin = 0
-
-        if rangeRound is not 'neither' and not getattr(self,'valueSteps',None):
-            self._valueMin, self._valueMax = valueMin, valueMax
-            P = self._calcTickPositions()
-            if len(P)>1:
-                valueStep = P[1]-P[0]
-            else:
-                oVS = self.valueStep
-                self.valueStep = None
-                P = self._calcTickPositions()
-                self.valueStep = oVS
-                if len(P)>1:
-                    valueStep = P[1]-P[0]
+                if self.valueMin is None and self.valueMax is None:
+                    zrp = getattr(self,'zrangePref',0)
+                    if zrp>0:
+                        valueMax = zrp
+                        valueMin = 0
+                    elif zrp<0:
+                        valueMax = 0
+                        valueMin = zrp
+                    else:
+                        valueMax = 0.01
+                        valueMin = -0.01
+                elif self.valueMin is None:
+                    valueMin = -0.01
                 else:
-                    valueStep = self._valueStep
-            fuzz = 1e-8*valueStep
-            if rangeRound in ['both','floor'] and valueMin<P[0]-fuzz: valueMin = P[0]-valueStep
-            if rangeRound in ['both','ceiling'] and valueMax>P[-1]+fuzz: valueMax = P[-1]+valueStep
+                    valueMax = 0.01
+            else:
+                if valueMax>0:
+                    valueMax = 1.2*valueMax
+                    valueMin = 0.0
+                else:
+                    valueMax = 0.0
+                    valueMin = 1.2*valueMin
+
+        forceZero = self.forceZero
+        if forceZero:
+            if forceZero=='near':
+                forceZero = min(abs(valueMin),abs(valueMax)) <= 5*(valueMax-valueMin)
+            if forceZero:
+                if valueMax<0: valueMax=0
+                elif valueMin>0: valueMin = 0
+
+        abf = self.avoidBoundFrac
+        do_rr = not getattr(self,'valueSteps',None)
+        do_abf = abf and do_rr
+        if type(abf) not in (TupleType,ListType):
+            abf = abf, abf
+        do_rr = rangeRound is not 'none' and do_rr
+        go = do_rr or do_abf
+        cache = {}
+        while go:
+            if do_abf:
+                valueStep, T, fuzz = self._getValueStepAndTicks(valueMin, valueMax, cache)
+                fuzz = 1e-8*valueStep
+                i0 = valueStep*abf[0]
+                i1 = valueStep*abf[1]
+                _n = getattr(self,'_cValueMin',T[0])
+                _x = getattr(self,'_cValueMax',T[-1])
+                if abs(T[0])>fuzz and _n - T[0] < i0-fuzz: valueMin = valueMin - i0
+                if abs(T[-1])>fuzz and T[-1]-_x < i1-fuzz: valueMax = valueMax + i1
+
+            go = 0
+            if do_rr:
+                valueStep, T, fuzz = self._getValueStepAndTicks(valueMin, valueMax, cache)
+                if rangeRound in ['both','floor'] and valueMin<T[0]-fuzz:
+                    valueMin = T[0]-valueStep
+                    go = 1
+                if rangeRound in ['both','ceiling'] and valueMax>T[-1]+fuzz:
+                    valueMax = T[-1]+valueStep
+                    go = 1
 
         self._valueMin, self._valueMax = valueMin, valueMax
         self._rangeAdjust()
 
-
     def _rangeAdjust(self):
         """Override this if you want to alter the calculated range.
 
         E.g. if want a minumamum range of 30% or don't want 100%
         as the first point.
         """
-
         pass
 
     def _adjustAxisTicks(self):
@@ -612,12 +677,9 @@ class ValueAxis(Widget):
 
     def _calcScaleFactor(self):
         """Calculate the axis' scale factor.
-
         This should be called only *after* the axis' range is set.
-
         Returns a number.
         """
-
         self._scaleFactor = self._length / float(self._valueMax - self._valueMin)
         return self._scaleFactor
 
@@ -635,34 +697,37 @@ class ValueAxis(Widget):
         return P
 
     def _calcTickmarkPositions(self):
-        """Calculate a list of tick positions on the axis.
-
-        Returns a list of numbers.
-        """
-
+        """Calculate a list of tick positions on the axis.  Returns a list of numbers."""
         self._tickValues = getattr(self,'valueSteps',None)
         if self._tickValues: return self._tickValues
-
         self._tickValues = self._calcTickPositions()
         self._adjustAxisTicks()
         return self._tickValues
 
     def _calcValueStep(self):
         '''Calculate _valueStep for the axis or get from valueStep.'''
-
         if self.valueStep is None:
             rawRange = self._valueMax - self._valueMin
             rawInterval = rawRange / min(float(self.maximumTicks-1),(float(self._length)/self.minimumTickSpacing))
-            niceInterval = nextRoundNumber(rawInterval)
-            self._valueStep = niceInterval
+            self._valueStep = nextRoundNumber(rawInterval)
         else:
             self._valueStep = self.valueStep
 
+    def _allIntTicks(self):
+        for tick in self._tickValues:
+            try:
+                if int(tick)!=tick: return 0
+            except:
+                return 0
+        return 1
 
     def makeTickLabels(self):
         g = Group()
 
-        f = self.labelTextFormat
+        f = self._labelTextFormat       # perhaps someone already set it
+        if f is None:
+            f = self.labelTextFormat or (self._allIntTicks() and '%d' or str)
+        elif f is str and self._allIntTicks(): f = '%d'
         pos = [self._x, self._y]
         d = self._dataIndex
         labels = self.labels
@@ -704,11 +769,18 @@ class ValueAxis(Widget):
                 L.strokeDashArray = strokeDashArray
                 g.add(L)
 
-    def makeGrid(self,g):
+    def makeGrid(self,g,dim=None):
         '''this is only called by a container object'''
-        if self.visibleGrid and (self.gridStart or self.gridEnd):
-            self._makeLines(g,self.gridStart,self.gridEnd,self.gridStrokeColor,self.gridStrokeWidth,self.gridStrokeDashArray)
-
+        s = self.gridStart
+        e = self.gridEnd
+        if dim:
+            s = s is None and dim[0]
+            e = e is None and dim[0]+dim[1]
+        c = self.gridStrokeColor
+        if self.visibleGrid and (s or e) and c is not None:
+            if self._dataIndex: offs = self._x
+            else: offs = self._y
+            self._makeLines(g,s-offs,e-offs,c,self.gridStrokeWidth,self.gridStrokeDashArray)
 
     def draw(self):
         g = Group()
@@ -955,9 +1027,7 @@ class NormalDateXValueAxis(XValueAxis):
                 if not isinstance(x,VC):
                     D[i] = (VC(x),y)
 
-    def configure(self, data):
-        self._convertXV(data)
-        xVals = map(lambda dv: dv[0], data[0])
+    def _getStepsAndLabels(self,xVals):
         if self.dailyFreq:
             xEOM = []
             pm = 0
@@ -973,18 +1043,25 @@ class NormalDateXValueAxis(XValueAxis):
             steps, labels = self._xAxisTicker(xEOM)
         else:
             steps, labels = self._xAxisTicker(xVals)
+        return steps, labels
+
+    def configure(self, data):
+        self._convertXV(data)
+        from reportlab.lib.set_ops import union
+        xVals = reduce(union,map(lambda x: map(lambda dv: dv[0],x),data),[])
+        xVals.sort()
+        steps,labels = self._getStepsAndLabels(xVals)
         valueMin, valueMax = self.valueMin, self.valueMax
         if valueMin is None: valueMin = xVals[0]
         if valueMax is None: valueMax = xVals[-1]
         self._valueMin, self._valueMax = valueMin, valueMax
         self._tickValues = steps
-        self.labelTextFormat = labels
+        self._labelTextFormat = labels
 
         self._scaleFactor = self._length / float(valueMax - valueMin)
         self._tickValues = steps
         self._configured = 1
 
-
 class YValueAxis(ValueAxis):
     "Y/value axis"
 
@@ -1128,7 +1205,7 @@ class AdjYValueAxis(YValueAxis):
 
         valueStep, requiredRange = self.valueStep, self.requiredRange
         if requiredRange and y_max - y_min < requiredRange:
-            y1, y2, None = find_good_grid(y_min, y_max,n=n,grid=valueStep)
+            y1, y2 = find_good_grid(y_min, y_max,n=n,grid=valueStep)[:2]
             if y2 - y1 < requiredRange:
                 ym = (y1+y2)*0.5
                 y1 = min(ym-requiredRange*0.5,y_min)
@@ -1144,18 +1221,25 @@ class AdjYValueAxis(YValueAxis):
         T, L = ticks(self._valueMin, self._valueMax, split=1, n=n, percent=self.leftAxisPercent,grid=valueStep)
         abf = self.avoidBoundFrac
         if abf:
+            i1 = (T[1]-T[0])
+            if type(abf) not in (TupleType,ListType):
+                i0 = i1 = i1*abf
+            else:
+                i0 = i1*abf[0]
+                i1 = i1*abf[1]
             _n = getattr(self,'_cValueMin',T[0])
             _x = getattr(self,'_cValueMax',T[-1])
-            i = (T[1]-T[0])*abf
-            if _n - T[0] < i: self._valueMin = self._valueMin - i
-            if T[-1]-_x < i: self._valueMax = self._valueMax + i
+            if _n - T[0] < i0: self._valueMin = self._valueMin - i0
+            if T[-1]-_x < i1: self._valueMax = self._valueMax + i1
             T, L = ticks(self._valueMin, self._valueMax, split=1, n=n, percent=self.leftAxisPercent,grid=valueStep)
 
         self._valueMin = T[0]
         self._valueMax = T[-1]
         self._tickValues = self.valueSteps = T
-        from reportlab.lib.formatters import DecimalFormatter
-        if not isinstance(self.labelTextFormat,DecimalFormatter): self.labelTextFormat = L
+        if self.labelTextFormat is None:
+            self._labelTextFormat = L
+        else:
+            self._labelTextFormat = self.labelTextFormat
 
         if abs(self._valueMin-100)<1e-6:
             self._calcValueStep()
