@@ -1,11 +1,11 @@
 #copyright ReportLab Inc. 2001
 #see license.txt for license details
 #history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/graphics/shapes.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/graphics/shapes.py,v 1.83 2002/11/27 20:19:24 rgbecker Exp $
+#$Header: /cvsroot/reportlab/reportlab/graphics/shapes.py,v 1.93 2003/07/02 15:37:28 johnprecedo Exp $
 """
 core of the graphics library - defines Drawing and Shapes
 """
-__version__=''' $Id: shapes.py,v 1.83 2002/11/27 20:19:24 rgbecker Exp $ '''
+__version__=''' $Id: shapes.py,v 1.93 2003/07/02 15:37:28 johnprecedo Exp $ '''
 
 import string, os, sys
 from math import pi, cos, sin, tan
@@ -321,11 +321,12 @@ class Group(Shape):
         """Return a new object which only contains primitive shapes."""
 
         # many limitations - shared nodes become multiple ones,
-        obj = self.__class__()
+        obj = isinstance(self,Drawing) and Drawing(self.width,self.height) or Group()
         obj._attrMap = self._attrMap.clone()
         if hasattr(obj,'transform'): obj.transform = self.transform[:]
 
         self_contents = self.contents
+        a = obj.contents.append
         for child in self_contents:
             if isinstance(child, UserNode):
                 newChild = child.provideNode()
@@ -333,7 +334,7 @@ class Group(Shape):
                 newChild = child.expandUserNodes()
             else:
                 newChild = child.copy()
-            obj.contents.append(newChild)
+            a(newChild)
 
         self._copyNamedContents(obj)
         return obj
@@ -367,9 +368,9 @@ class Group(Shape):
         self_contents = self.contents
         if not aKeys: aKeys = self._attrMap.keys()
         for (k, v) in self.__dict__.items():
-            if k in self_contents:
+            if v in self_contents:
                 pos = self_contents.index(v)
-                setattr(obj, oldKey, obj.contents[pos])
+                setattr(obj, k, obj.contents[pos])
             elif k in aKeys and k not in noCopy:
                 setattr(obj, k, copy(v))
 
@@ -546,9 +547,11 @@ class Drawing(Group, Flowable):
     def asGroup(self,*args,**kw):
         return self._copy(apply(Group,args,kw))
 
-    def save(self, formats=None, verbose=None, fnRoot=None, outDir=None):
+    def save(self, formats=None, verbose=None, fnRoot=None, outDir=None, title=''):
+        """Saves copies of self in desired location and formats.
+
+        Multiple formats can be supported in one call"""
         from reportlab import rl_config
-        "Saves copies of self in desired location and formats"
         ext = ''
         if not fnRoot:
             fnRoot = getattr(self,'fileNamePattern',(self.__class__.__name__+'%03d'))
@@ -569,7 +572,7 @@ class Drawing(Group, Flowable):
         if not os.path.isdir(outDir): os.makedirs(outDir)
         fnroot = os.path.normpath(os.path.join(outDir,fnRoot))
         plotMode = os.path.splitext(fnroot)
-        if string.lower(plotMode[1][1:]) in ['pdf','ps','eps','gif','png','jpg','jpeg','tiff','tif','py']:
+        if string.lower(plotMode[1][1:]) in ['pdf','ps','eps','gif','png','jpg','jpeg','pct','pict','tiff','tif','py']:
             fnroot = plotMode[0]
 
         plotMode, verbose = formats or getattr(self,'formats',['pdf']), (verbose is not None and (verbose,) or (getattr(self,'verbose',verbose),))[0]
@@ -579,36 +582,20 @@ class Drawing(Group, Flowable):
             from reportlab.graphics import renderPDF
             filename = fnroot+'.pdf'
             if verbose: print "generating PDF file %s" % filename
-            renderPDF.drawToFile(self, filename, fnroot,showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
+            renderPDF.drawToFile(self, filename, title, showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
             ext = ext +  '/.pdf'
+            if sys.platform=='mac':
+                import macfs, macostools
+                macfs.FSSpec(filename).SetCreatorType("CARO", "PDF ")
+                macostools.touched(filename)
 
-        if 'gif' in plotMode:
-            from reportlab.graphics import renderPM
-            filename = fnroot+'.gif'
-            if verbose: print "generating GIF file %s" % filename
-            renderPM.drawToFile(self, filename,fmt='GIF',showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
-            ext = ext +  '/.gif'
-
-        if 'png' in plotMode:
-            from reportlab.graphics import renderPM
-            filename = fnroot+'.png'
-            if verbose: print "generating PNG file %s" % filename
-            renderPM.drawToFile(self, filename,fmt='PNG',showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
-            ext = ext +  '/.png'
-
-        if 'tif' in plotMode:
-            from reportlab.graphics import renderPM
-            filename = fnroot+'.tif'
-            if verbose: print "generating TIFF file %s" % filename
-            renderPM.drawToFile(self, filename,fmt='TIFF',showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
-            ext = ext +  '/.tif'
-
-        if 'jpg' in plotMode:
-            from reportlab.graphics import renderPM
-            filename = fnroot+'.jpg'
-            if verbose: print "generating JPG file %s" % filename
-            renderPM.drawToFile(self, filename,fmt='JPEG',showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
-            ext = ext +  '/.jpg'
+        for bmFmt in ['gif','png','tif','jpg','tiff','pct','pict']:
+            if bmFmt in plotMode:
+                from reportlab.graphics import renderPM
+                filename = '%s.%s' % (fnroot,bmFmt)
+                if verbose: print "generating %s file %s" % (bmFmt,filename)
+                renderPM.drawToFile(self, filename,fmt=bmFmt,showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
+                ext = ext + '/.' + bmFmt
 
         if 'eps' in plotMode:
             from rlextra.graphics import renderPS_SEP
@@ -641,6 +628,29 @@ class Drawing(Group, Flowable):
             self.saveLogger(fnroot,ext)
         return ext and fnroot+ext[1:] or ''
 
+
+    def asString(self, format, verbose=None):
+        """Converts to an 8 bit string in given format."""
+        assert format in ['pdf','ps','eps','gif','png','jpg','jpeg','bmp','ppm','tiff','tif','py','pict','pct'], 'Unknown file format "%s"' % format
+        from reportlab import rl_config
+        #verbose = verbose is not None and (verbose,) or (getattr(self,'verbose',verbose),)[0]
+        if format == 'pdf':
+            from reportlab.graphics import renderPDF
+            return renderPDF.drawToString(self)
+        elif format in ['gif','png','tif','jpg','pct','pict','bmp','ppm']:
+            from reportlab.graphics import renderPM
+            return renderPM.drawToString(self, fmt=format)
+        elif format == 'eps':
+            from rlextra.graphics import renderPS_SEP
+            return renderPS_SEP.drawToString(self,
+                                preview = getattr(self,'preview',1),
+                                showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
+        elif format == 'ps':
+            from reportlab.graphics import renderPS
+            return renderPS.drawToString(self, showBoundary=getattr(self,'showBorder',rl_config.showBoundary))
+        elif format == 'py':
+            return self._renderPy()
+
 class _DrawingEditorMixin:
     '''This is a mixin to provide functionality for edited drawings'''
     def _add(self,obj,value,name=None,validate=None,desc=None,pos=None):
@@ -908,12 +918,13 @@ class Wedge(SolidShape):
         startangledegrees = AttrMapValue(isNumber),
         endangledegrees = AttrMapValue(isNumber),
         yradius = AttrMapValue(isNumberOrNone),
+        radius1 = AttrMapValue(isNumberOrNone),
+        yradius1 = AttrMapValue(isNumberOrNone),
         )
 
     degreedelta = 1 # jump every 1 degrees
 
     def __init__(self, centerx, centery, radius, startangledegrees, endangledegrees, yradius=None, **kw):
-        if yradius is None: yradius = radius
         SolidShape.__init__(self, kw)
         while endangledegrees<startangledegrees:
             endangledegrees = endangledegrees+360
@@ -922,39 +933,53 @@ class Wedge(SolidShape):
             centerx, centery, radius, startangledegrees, endangledegrees
         self.yradius = yradius
 
+    def _xtraRadii(self):
+        yradius = getattr(self, 'yradius', None)
+        if yradius is None: yradius = self.radius
+        radius1 = getattr(self,'radius1', None)
+        yradius1 = getattr(self,'yradius1',radius1)
+        if radius1 is None: radius1 = yradius1
+        return yradius, radius1, yradius1
+
     #def __repr__(self):
     #        return "Wedge"+repr((self.centerx, self.centery, self.radius, self.startangledegrees, self.endangledegrees ))
     #__str__ = __repr__
 
     def asPolygon(self):
         #print "asPolygon"
-        centerx, centery, radius, startangledegrees, endangledegrees = \
-            self.centerx, self.centery, self.radius, self.startangledegrees, self.endangledegrees
-        yradius = self.yradius
+        centerx= self.centerx
+        centery = self.centery
+        radius = self.radius
+        yradius, radius1, yradius1 = self._xtraRadii()
+        startangledegrees = self.startangledegrees
+        endangledegrees = self.endangledegrees
         degreedelta = self.degreedelta
         points = []
         a = points.append
-        a(centerx); a(centery)
         from math import sin, cos, pi
         degreestoradians = pi/180.0
         radiansdelta = degreedelta*degreestoradians
         startangle = startangledegrees*degreestoradians
         endangle = endangledegrees*degreestoradians
         while endangle<startangle:
-            #print "endangle", endangle
             endangle = endangle+2*pi
         angle = startangle
-        #print "start", startangle, "end", endangle
+        CA = []
+        CAA = CA.append
         while angle<endangle:
-            #print angle
-            x = centerx + cos(angle)*radius
-            y = centery + sin(angle)*yradius
-            a(x); a(y)
+            CAA((cos(angle),sin(angle)))
             angle = angle+radiansdelta
-        #print "done"
-        x = centerx + cos(endangle)*radius
-        y = centery + sin(endangle)*yradius
-        a(x); a(y)
+        CAA((cos(endangle),sin(endangle)))
+        for c,s in CA:
+            a(centerx+radius*c)
+            a(centery+yradius*s)
+        if (radius1==0 or radius1 is None) and (yradius1==0 or yradius1 is None):
+            a(centerx); a(centery)
+        else:
+            CA.reverse()
+            for c,s in CA:
+                a(centerx+radius1*c)
+                a(centery+yradius1*s)
         return Polygon(points)
 
     def copy(self):
