@@ -2,12 +2,12 @@
 #copyright ReportLab Inc. 2000
 #see license.txt for license details
 #history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/lib/_rl_accel.c?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/lib/_rl_accel.c,v 1.30 2002/11/20 17:35:52 rgbecker Exp $
+#$Header: /cvsroot/reportlab/reportlab/lib/_rl_accel.c,v 1.34 2003/05/16 09:39:20 johnprecedo Exp $
  ****************************************************************************/
 #if 0
-static __version__=" $Id: _rl_accel.c,v 1.30 2002/11/20 17:35:52 rgbecker Exp $ "
+static __version__=" $Id: _rl_accel.c,v 1.34 2003/05/16 09:39:20 johnprecedo Exp $ "
 #endif
-#include <Python.h>
+#include "Python.h"
 #include <stdlib.h>
 #include <math.h>
 #if defined(__GNUC__) || defined(sun) || defined(_AIX) || defined(__hpux)
@@ -27,7 +27,7 @@ static __version__=" $Id: _rl_accel.c,v 1.30 2002/11/20 17:35:52 rgbecker Exp $
 #ifndef min
 #	define min(a,b) ((a)<(b)?(a):(b))
 #endif
-#define VERSION "0.40"
+#define VERSION "0.42"
 #define MODULE "_rl_accel"
 #ifndef	ATTRDICT
 	#if PY_MAJOR_VERSION>=2
@@ -396,9 +396,9 @@ PyObject *_a85_decode(PyObject *self, PyObject *args)
 	PyObject		*retVal;
 
 	if (!PyArg_ParseTuple(args, "z#", &inData, &length)) return NULL;
-	for(k=0,q=inData, p=q+length;q<p && (q=strchr(q,'z'));k++, q++);	/*count 'z'*/
+	for(k=0,q=inData, p=q+length;q<p && (q=(unsigned char*)strchr((const char*)q,'z'));k++, q++);	/*count 'z'*/
 	length += k*4;
-	tmp = q = (char*)malloc(length+1);
+	tmp = q = (unsigned char*)malloc(length+1);
 	while(inData<p && (k = *inData++)){
 		if(isspace(k)) continue;
 		if(k=='z'){
@@ -423,7 +423,7 @@ PyObject *_a85_decode(PyObject *self, PyObject *args)
 	blocks = length / 5;
 	extra = length % 5;
 
-	buf = (char*)malloc((blocks+1)*4);
+	buf = (unsigned char*)malloc((blocks+1)*4);
 	q = inData+blocks*5;
 	for(k=0;inData<q;inData+=5){
 		c1 = inData[0]-33;
@@ -454,7 +454,7 @@ PyObject *_a85_decode(PyObject *self, PyObject *args)
 				}
 			}
 		}
-	retVal = PyString_FromStringAndSize(buf, k);
+	retVal = PyString_FromStringAndSize((const char*)buf, k);
 	free(buf);
 	free(tmp);
 	return retVal;
@@ -467,7 +467,10 @@ static	char *_fp_one(PyObject *pD)
 	static	char s[30];
 	int l;
 	char*	dot;
-	if((pD=PyNumber_Float(pD))) d = PyFloat_AS_DOUBLE(pD);
+	if((pD=PyNumber_Float(pD))){
+		d = PyFloat_AS_DOUBLE(pD);
+		Py_DECREF(pD);
+		}
 	else {
 		PyErr_SetString(ErrorObject, "bad numeric value");
 		return NULL;
@@ -511,11 +514,18 @@ PyObject *_fp_str(PyObject *self, PyObject *args)
 				args = retVal;
 				}
 			else PyErr_Clear();
+			Py_DECREF(retVal);
 			}
 		buf=malloc(31*aL);
 		pB = buf;
 		for(i=0;i<aL;i++){
-			if(!(pD = _fp_one(PySequence_GetItem(args,i)))){
+			retVal = PySequence_GetItem(args,i);
+			if(retVal){
+				pD = _fp_one(retVal);
+				Py_DECREF(retVal);
+				}
+			else pD = NULL;
+			if(!pD){
 				free(buf);
 				return NULL;
 				}
@@ -705,7 +715,7 @@ static PyObject *_instanceEscapePDF(PyObject *unused, PyObject* args)
 static PyObject *_sameFrag(PyObject *self, PyObject* args)
 {
 	PyObject *f, *g;
-	static char *names[] = {"fontName", "fontSize", "textColor", "rise", NULL};
+	static char *names[] = {"fontName", "fontSize", "textColor", "rise", "underline", NULL};
 	int	r=0, t;
 	char **p;
 	if (!PyArg_ParseTuple(args, "OO:_sameFrag", &f, &g)) return NULL;
