@@ -1,8 +1,8 @@
 #copyright ReportLab Inc. 2000
 #see license.txt for license details
 #history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/pdfbase/pdfdoc.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/pdfbase/pdfdoc.py,v 1.75 2002/11/28 18:36:46 rgbecker Exp $
-__version__=''' $Id: pdfdoc.py,v 1.75 2002/11/28 18:36:46 rgbecker Exp $ '''
+#$Header: /cvsroot/reportlab/reportlab/pdfbase/pdfdoc.py,v 1.83 2003/07/08 22:15:17 rgbecker Exp $
+__version__=''' $Id: pdfdoc.py,v 1.83 2003/07/08 22:15:17 rgbecker Exp $ '''
 __doc__="""
 The module pdfdoc.py handles the 'outer structure' of PDF documents, ensuring that
 all objects are properly cross-referenced and indexed to the nearest byte.  The
@@ -76,18 +76,6 @@ Pages = "Pages"
 # for % substitutions
 LINEENDDICT = {"LINEEND": LINEEND, "PERCENT": "%"}
 
-def markfilename(filename):
-    # with the Mac, we need to tag the file in a special
-    #way so the system knows it is a PDF file.
-    #This supplied by Joe Strout
-    import os
-    if os.name == 'mac':
-        import macfs
-        try:
-            macfs.FSSpec(filename).SetCreatorType('CARO','PDF ')
-        except:
-            pass
-
 def format(element, document, toplevel=0):
     """Indirection step for formatting.
        Ensures that document parameters alter behaviour
@@ -151,7 +139,11 @@ class PDFDocument:
     defaultStreamFilters = None
     encrypt = NoEncryption() # default no encryption
     pageCounter = 1
-    def __init__(self, encoding=rl_config.defaultEncoding, dummyoutline=0,compression=rl_config.pageCompression):
+    def __init__(self,
+                 encoding=rl_config.defaultEncoding,
+                 dummyoutline=0,
+                 compression=rl_config.pageCompression,
+                 invariant=rl_config.invariant):
         #self.defaultStreamFilters = [PDFBase85Encode, PDFZCompress] # for testing!
         #self.defaultStreamFilters = [PDFZCompress] # for testing!
         assert encoding in ['MacRomanEncoding',
@@ -161,13 +153,22 @@ class PDFDocument:
         if encoding[-8:] <> 'Encoding':
             encoding = encoding + 'Encoding'
 
+        # allow None value to be passed in to mean 'give system defaults'
+        if invariant is None:
+            self.invariant = rl_config.invariant
+        else:
+            self.invariant = invariant
         self.setCompression(compression)
         self.encoding = encoding
         # signature for creating PDF ID
-        import md5, time
+        import md5
         sig = self.signature = md5.new()
         sig.update("a reportlab document")
-        sig.update(repr(time.time())) # initialize with timestamp digest
+        if not self.invariant:
+            cat = _getTimeStamp()
+        else:
+            cat = 946684800.0
+        sig.update(repr(cat)) # initialize with timestamp digest
         # mapping of internal identifier ("Page001") to PDF objectnumber and generation number (34, 0)
         self.idToObjectNumberAndVersion = {}
         # mapping of internal identifier ("Page001") to PDF object (PDFPage instance)
@@ -186,6 +187,7 @@ class PDFDocument:
         self.Outlines = self.outline = outlines
         cat.Outlines = outlines
         self.info = PDFInfo()
+        self.info.invariant = self.invariant
         #self.Reference(self.Catalog)
         #self.Reference(self.Info)
         self.fontMapping = {}
@@ -205,6 +207,7 @@ class PDFDocument:
         self.signature.update(str(thing))
 
     def ID(self):
+        "A unique fingerprint for the file (unless in invariant mode)"
         if self._ID:
             return self._ID
         digest = self.signature.digest()
@@ -212,21 +215,10 @@ class PDFDocument:
         ID = PDFString(digest)
         IDs = ID.format(doc)
         self._ID = "%s %% ReportLab generated PDF document -- digest (http://www.reportlab.com) %s [%s %s] %s" % (
-            LINEEND, LINEEND, IDs, IDs, LINEEND)
+                LINEEND, LINEEND, IDs, IDs, LINEEND)
         return self._ID
 
     def SaveToFile(self, filename, canvas):
-        # realize delayed fonts
-        for fnt in self.delayedFonts:
-            fnt.addObjects(self)
-        # add info stuff to signature
-        self.info.digest(self.signature)
-        ### later: maybe add more info to sig?
-        # prepare outline
-        self.Reference(self.Catalog)
-        self.Reference(self.info)
-        outline = self.outline
-        outline.prepare(self, canvas)
         if callable(getattr(filename, "write",None)):
             myfile = 0
             f = filename
@@ -235,13 +227,28 @@ class PDFDocument:
             myfile = 1
             filename = str(filename)
             f = open(filename, "wb")
-        txt = self.format()
-        f.write(txt)
+        f.write(self.GetPDFData(canvas))
         if myfile:
             f.close()
+            from reportlab.lib.utils import markfilename
             markfilename(filename) # do platform specific file junk
         if getattr(canvas,'_verbosity',None): print 'saved', filename
 
+    def GetPDFData(self, canvas):
+        # realize delayed fonts
+        for fnt in self.delayedFonts:
+            fnt.addObjects(self)
+        # add info stuff to signature
+        self.info.invariant = self.invariant
+        self.info.digest(self.signature)
+        ### later: maybe add more info to sig?
+        # prepare outline
+        self.Reference(self.Catalog)
+        self.Reference(self.info)
+        outline = self.outline
+        outline.prepare(self, canvas)
+        return self.format()
+
     def inPage(self):
         """specify the current object as a page (enables reference binding and other page features)"""
         if self.inObject is not None:
@@ -366,7 +373,11 @@ class PDFDocument:
                 IOf = IO.format(self)
                 # add a comment to the PDF output
                 if DoComments:
-                    File.add("%% %s %s %s" % (repr(id), repr(repr(obj)[:50]), LINEEND))
+                    try:
+                        classname = obj.__class__.__name__
+                    except:
+                        classname = repr(obj)
+                    File.add("%% %s: class %s %s" % (repr(id), classname[:50], LINEEND))
                 offset = File.add(IOf)
                 idToOf[id] = offset
                 ids.append(id)
@@ -1286,11 +1297,11 @@ class PDFInfo:
     File | Document Info in Acrobat Reader.  If this is wrong, you get
     Postscript errors while printing, even though it does not print."""
     def __init__(self):
+        self.invariant = rl_config.invariant
         self.title = "untitled"
         self.author = "anonymous"
         self.subject = "unspecified"
-        #now = time.localtime(time.time())
-        #self.datestr = '%04d%02d%02d%02d%02d%02d' % tuple(now[0:6])
+
     def digest(self, md5object):
         # add self information to signature
         for x in (self.title, self.author, self.subject):
@@ -1300,7 +1311,7 @@ class PDFInfo:
         D = {}
         D["Title"] = PDFString(self.title)
         D["Author"] = PDFString(self.author)
-        D["CreationDate"] = PDFDate()
+        D["CreationDate"] = PDFDate(invariant=self.invariant)
         D["Producer"] = PDFString("ReportLab http://www.reportlab.com")
         D["Subject"] = PDFString(self.subject)
         PD = PDFDictionary(D)
@@ -1402,16 +1413,33 @@ class PDFRectangle:
         A = PDFArray([self.llx, self.lly, self.ulx, self.ury])
         return format(A, document)
 
-DATEFMT = '%04d%02d%02d%02d%02d%02d'
-import time
-(nowyyyy, nowmm, nowdd, nowhh, nowm, nows) = tuple(time.localtime(time.time())[:6])
+_NOWT=None
+def _getTimeStamp():
+    global _NOWT
+    if not _NOWT:
+        import time
+        _NOWT = time.time()
+    return _NOWT
 
 class PDFDate:
     # gmt offset not yet suppported
-    def __init__(self, yyyy=nowyyyy, mm=nowmm, dd=nowdd, hh=nowhh, m=nowm, s=nows):
+    def __init__(self, yyyy=None, mm=None, dd=None, hh=None, m=None, s=None, invariant=rl_config.invariant):
+        if None in (yyyy, mm, dd, hh, m, s):
+            if invariant:
+                now = (2000,01,01,00,00,00,0)
+            else:
+                import time
+                now = tuple(time.localtime(_getTimeStamp())[:6])
+        if yyyy is None: yyyy=now[0]
+        if mm is None: mm=now[1]
+        if dd is None: dd=now[2]
+        if hh is None: hh=now[3]
+        if m is None: m=now[4]
+        if s is None: s=now[5]
         self.yyyy=yyyy; self.mm=mm; self.dd=dd; self.hh=hh; self.m=m; self.s=s
+
     def format(self, doc):
-        S = PDFString(DATEFMT % (self.yyyy, self.mm, self.dd, self.hh, self.m, self.s))
+        S = PDFString('%04d%02d%02d%02d%02d%02d' % (self.yyyy, self.mm, self.dd, self.hh, self.m, self.s))
         return format(S, doc)
 
 
@@ -1461,7 +1489,10 @@ class Destination:
 class PDFDestinationXYZ:
     typename = "XYZ"
     def __init__(self, page, left, top, zoom):
-        self.page = page; self.top=top; self.zoom=zoom
+        self.page = page
+        self.top = top
+        self.zoom = zoom
+        self.left = left
     def format(self, document):
         pageref = document.Reference(self.page)
         A = PDFArray( [ pageref, PDFName(self.typename), self.left, self.top, self.zoom ] )
@@ -1742,7 +1773,7 @@ class PDFImageXObject:
         self.width, self.height = map(string.atoi,(words[1],words[3]))
         self.colorSpace = 'DeviceRGB'
         self.bitsPerComponent = 8
-        self._filters = 'A85','Fl'
+        self._filters = 'ASCII85Decode','FlateDecode' #'A85','Fl'
         if IMG: self._checkTransparency(IMG[0])
         else: self.mask = None
         self.streamContent = string.join(imagedata[3:-1],'')
@@ -1759,17 +1790,24 @@ class PDFImageXObject:
             self.colorSpace = 'DeviceCMYK'
         imageFile.seek(0) #reset file pointer
         self.streamContent = pdfutils._AsciiBase85Encode(imageFile.read())
-        self._filters = 'A85','DCT'
+        self._filters = 'ASCII85Decode','DCTDecode' #'A85','DCT'
         self.mask = None
 
     def _checkTransparency(self,PILImage):
         if self.mask=='auto':
             if PILImage.info.has_key("transparency") :
                 transparency = PILImage.info["transparency"] * 3
-                (tred, tgreen, tblue) = map(ord, PILImage.palette.data[transparency:transparency+3])
+                palette = PILImage.palette
+                try:
+                    palette = palette.palette
+                except:
+                    palette = palette.data
+                (tred, tgreen, tblue) = map(ord, palette[transparency:transparency+3])
                 self.mask = (tred, tred, tgreen, tgreen, tblue, tblue)
             else:
                 self.mask = None
+        elif hasattr(self.mask,'rgb'):
+            self.mask = self.mask.rgb()+self.mask.rgb()
 
     def loadImageFromPIL(self, PILImage):
         "Extracts the stream, width and height"
@@ -1785,7 +1823,7 @@ class PDFImageXObject:
         self.streamContent = pdfutils._AsciiBase85Encode(zlib.compress(raw))
         self.colorSpace = 'DeviceRGB'
         self.bitsPerComponent = 8
-        self._filters = 'A85','Fl'  # Ascii85decode, FlateDecode
+        self._filters = 'ASCII85Decode','FlateDecode' #'A85','Fl'
         self._checkTransparency(PILImage)
 
     def format(self, document):
