@@ -1,19 +1,20 @@
 #copyright ReportLab Inc. 2000
 #see license.txt for license details
 #history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/pdfgen/canvas.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/pdfgen/canvas.py,v 1.108 2002/11/06 11:32:42 rgbecker Exp $
-__version__=''' $Id: canvas.py,v 1.108 2002/11/06 11:32:42 rgbecker Exp $ '''
+#$Header: /cvsroot/reportlab/reportlab/pdfgen/canvas.py,v 1.114 2003/07/03 14:20:04 rgbecker Exp $
+__version__=''' $Id: canvas.py,v 1.114 2003/07/03 14:20:04 rgbecker Exp $ '''
 __doc__="""
 The Canvas object is the primary interface for creating PDF files. See
 doc/userguide.pdf for copious examples.
 """
 
+
+
 ENABLE_TRACKING = 1 # turn this off to do profile testing w/o tracking
 
 import os
 import sys
 from string import join, split, strip, atoi, replace, upper
-import time
 import tempfile
 from types import *
 from math import sin, cos, tan, pi, ceil
@@ -115,13 +116,17 @@ class Canvas:
                  bottomup = 1,
                  pageCompression=None,
                  encoding=rl_config.defaultEncoding,
+                 invariant=rl_config.invariant,
                  verbosity=0):
         """Create a canvas of a given size. etc.
         Most of the attributes are private - we will use set/get methods
         as the preferred interface.  Default page size is A4."""
         self._filename = filename
         self._encodingName = encoding
-        self._doc = pdfdoc.PDFDocument(encoding,compression=pageCompression)
+        self._doc = pdfdoc.PDFDocument(encoding,
+                                       compression=pageCompression,
+                                       invariant=invariant)
+
 
         #this only controls whether it prints 'saved ...' - 0 disables
         self._verbosity = verbosity
@@ -130,6 +135,7 @@ class Canvas:
         self._onPage = None
 
         self._pagesize = pagesize
+        self._pageRotation = 0
         #self._currentPageHasImages = 0
         self._pageTransition = None
         self._pageDuration = None
@@ -357,6 +363,7 @@ class Canvas:
         page = pdfdoc.PDFPage()
         page.pagewidth = self._pagesize[0]
         page.pageheight = self._pagesize[1]
+        page.Rotate = self._pageRotation
         page.hasImages = self._currentPageHasImages
         page.setPageTransition(self._pageTransition)
         page.setCompression(self._pageCompression)
@@ -404,15 +411,80 @@ class Canvas:
             result = d[name] = pdfdoc.Destination(name) # newly defined, unbound
         return result
 
-    def bookmarkPage(self, key):
-        """bind a bookmark (destination) to the current page"""
-        # XXXX there are a lot of other ways a bookmark destination can be bound: should be implemented.
-        # XXXX the other ways require tracking of the graphics state....
+    def bookmarkPage(self, key,
+                      fitType="Fit",
+                      left=None,
+                      top=None,
+                      bottom=None,
+                      right=None,
+                      zoom=None
+                      ):
+        """
+        This creates a bookmark to the current page which can
+        be referred to with the given key elsewhere.
+
+        PDF offers very fine grained control over how Acrobat
+        reader is zoomed when people link to this. The default
+        is to keep the user's current zoom settings. the last
+        arguments may or may not be needed depending on the
+        choice of 'fitType'.
+
+        Fit types and the other arguments they use are:
+        /XYZ left top zoom - fine grained control.  null
+          or zero for any of the parameters means 'leave
+          as is', so "0,0,0" will keep the reader's settings.
+          NB. Adobe Reader appears to prefer "null" to 0's.
+
+        /Fit - entire page fits in window
+
+        /FitH top - top coord at top of window, width scaled
+                    to fit.
+
+        /FitV left - left coord at left of window, height
+                     scaled to fit
+
+        /FitR left bottom right top - scale window to fit
+                                  the specified rectangle
+
+        (question: do we support /FitB, FitBH and /FitBV
+        which are hangovers from version 1.1 / Acrobat 3.0?)"""
         dest = self._bookmarkReference(key)
         self._doc.inPage() # try to enable page-only features
         pageref = self._doc.thisPageRef()
-        dest.fit()
-        dest.setPage(pageref) # formatter won't make a ref to a ref
+
+        #None = "null" for PDF
+        if left is None:
+            left = "null"
+        if top is None:
+            top = "null"
+        if bottom is None:
+            bottom = "null"
+        if right is None:
+            right = "null"
+        if zoom is None:
+            zoom = "null"
+
+        if fitType == "XYZ":
+            dest.xyz(left,top,zoom)
+        elif fitType == "Fit":
+            dest.fit()
+        elif fitType == "FitH":
+            dest.fith(top)
+        elif fitType == "FitV":
+            dest.fitv(left)
+        elif fitType == "FitR":
+            dest.fitr(left,bottom,right,top)
+        #Do we need these (version 1.1 / Acrobat 3 versions)?
+        elif fitType == "FitB":
+            dest.fitb()
+        elif fitType == "FitBH":
+            dest.fitbh(top)
+        elif fitType == "FitBV":
+            dest.fitbv(left)
+        else:
+            raise "Unknown Fit type %s" % (fitType,)
+
+        dest.setPage(pageref)
         return dest
 
     def bookmarkHorizontalAbsolute(self, key, yhorizontal):
@@ -423,12 +495,8 @@ class Canvas:
            etcetera) in effect for the current graphics state.  The programmer is
            responsible for making sure the bookmark matches an appropriate item on
            the page."""
-        dest = self._bookmarkReference(key)
-        self._doc.inPage() # try to enable page-only features
-        pageref = self._doc.thisPageRef()
-        dest.fith(yhorizontal)
-        dest.setPage(pageref)
-        return dest
+        #This method should probably be deprecated since it is just a sub-set of bookmarkPage
+        return self.bookmarkPage(key,fitType="FitH",top=yhorizontal)
 
     def bookmarkHorizontal(self, key, relativeX, relativeY):
         """w.r.t. the current transformation, bookmark this horizontal."""
@@ -749,12 +817,24 @@ class Canvas:
         if len(self._code): self.showPage()
         self._doc.SaveToFile(self._filename, self)
 
+    def getpdfdata(self):
+        """Returns the PDF data that would normally be written to a file.
+        If there is current data a ShowPage is executed automatically.
+        After this operation the canvas must not be used further."""
+        if len(self._code): self.showPage()
+        return self._doc.GetPDFData(self)
+
     def setPageSize(self, size):
         """accepts a 2-tuple in points for paper size for this
         and subsequent pages"""
         self._pagesize = size
         self._make_preamble()
 
+    def setPageRotation(self, rot):
+        """Instruct display device that this page is to be rotated"""
+        assert rot % 90.0 == 0.0, "Rotation must be a multiple of 90 degrees"
+        self._pageRotation = rot
+
     def addLiteral(self, s, escaped=1):
         """introduce the literal text of PDF operations s into the current stream.
            Only use this if you are an expert in the PDF file format."""
