@@ -1,9 +1,9 @@
 #copyright ReportLab Inc. 2000
 #see license.txt for license details
 #history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/platypus/doctemplate.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/platypus/doctemplate.py,v 1.58 2003/01/02 11:30:05 rgbecker Exp $
+#$Header: /cvsroot/reportlab/reportlab/platypus/doctemplate.py,v 1.63 2003/05/11 11:01:30 rgbecker Exp $
 
-__version__=''' $Id: doctemplate.py,v 1.58 2003/01/02 11:30:05 rgbecker Exp $ '''
+__version__=''' $Id: doctemplate.py,v 1.63 2003/05/11 11:01:30 rgbecker Exp $ '''
 
 __doc__="""
 This module contains the core structure of platypus.
@@ -137,6 +137,21 @@ class _FrameBreak(ActionFlowable):
 FrameBreak = _FrameBreak('frameEnd')
 PageBegin = ActionFlowable('pageBegin')
 
+class Indenter(ActionFlowable):
+    """Increases or decreases left and right margins of frame.
+
+    This allows one to have a 'context-sensitive' indentation
+    and makes nested lists way easier.
+    """    
+
+    def __init__(self, left=0, right=0):
+        self.left = left
+        self.right = right
+
+    def apply(self, doc):
+        doc.frame._leftExtraIndent = doc.frame._leftExtraIndent + self.left
+        doc.frame._rightExtraIndent = doc.frame._rightExtraIndent + self.right
+        
 
 class NextPageTemplate(ActionFlowable):
     """When you get to the next page, use the template specified (change to two column, for example)  """
@@ -151,7 +166,7 @@ class PageTemplate:
     derived classes can also implement beforeDrawPage and afterDrawPage if they want
     """
     def __init__(self,id=None,frames=[],onPage=_doNothing, onPageEnd=_doNothing,
-                 pagesize=defaultPageSize):
+                 pagesize=None):
         if type(frames) not in (ListType,TupleType): frames = [frames]
         assert filter(lambda x: not isinstance(x,Frame), frames)==[], "frames argument error"
         self.id = id
@@ -168,7 +183,7 @@ class PageTemplate:
 
     def checkPageSize(self,canv,doc):
         '''This gets called by the template framework
-        If canv size != doc size then the canv size is set to
+        If canv size != template size then the canv size is set to
         the template size or if that's not available to the
         doc size.
         '''
@@ -255,6 +270,7 @@ class BaseDocTemplate:
                     'allowSplitting':1,
                     'title':None,
                     'author':None,
+                    'invariant':None,
                     '_pageBreakQuick':1}
     _invalidInitArgs = ()
 
@@ -289,7 +305,10 @@ class BaseDocTemplate:
         self._onProgress = None
         self._flowableCount = 0  # so we know how far to go
 
-
+        #context sensitive margins - set by story, not from outside
+        self._leftExtraIndent = 0.0
+        self._rightExtraIndent = 0.0
+        
         self._calc()
         self.afterInit()
 
@@ -373,11 +392,17 @@ class BaseDocTemplate:
         if f._atTop:
             if self.showBoundary or self.frame.showBoundary:
                 self.frame.drawBoundary(self.canv)
+        f._leftExtraIndent = self._leftExtraIndent
+        f._rightExtraIndent = self._rightExtraIndent
 
     def handle_frameEnd(self,resume=0):
         ''' Handles the semantics of the end of a frame. This includes the selection of
             the next frame or if this is the last frame then invoke pageEnd.
         '''
+        
+        self._leftExtraIndent = self.frame._leftExtraIndent
+        self._rightExtraIndent = self.frame._rightExtraIndent
+
         if hasattr(self,'_nextFrameIndex'):
             frame = self.pageTemplate.frames[self._nextFrameIndex]
             del self._nextFrameIndex
@@ -500,6 +525,7 @@ class BaseDocTemplate:
                 if self.allowSplitting:
                     # see if this is a splittable thing
                     S = self.frame.split(f,self.canv)
+                    #print '%d parts to sequence on page %d' % (len(S), self.page)
                     n = len(S)
                 else:
                     n = 0
@@ -522,8 +548,8 @@ class BaseDocTemplate:
                         #HACK = it seems within tables we sometimes
                         #get an empty paragraph that won't fit and this
                         #causes it to fall over.  FIXME FIXME FIXME
-                        raise "LayoutError", message
-##                  f.postponed = 1
+                        #raise "LayoutError", message
+##                    f.postponed = 1
                     f._postponed = 1
                     flowables.insert(0,f)           # put the flowable back
                     self.handle_frameEnd()
@@ -541,7 +567,9 @@ class BaseDocTemplate:
 
     def _startBuild(self, filename=None, canvasmaker=canvas.Canvas):
         self._calc()
-        self.canv = canvasmaker(filename or self.filename,pagesize=self.pagesize)
+        self.canv = canvasmaker(filename or self.filename,
+                                pagesize=self.pagesize,
+                                invariant=self.invariant)
         if self._onPage:
             self.canv.setPageCallBack(self._onPage)
         self.handle_documentBegin()
@@ -618,22 +646,10 @@ class BaseDocTemplate:
         #print 'scanned story, found these indexing flowables:\n'
         #print self._indexingFlowables
 
-        # attempt to fix the filename is a 'file' problem
-        isFile = getattr(filename,'write',None)
-        if isFile:
-            hasSeek = getattr(filename,'seek',None)
-            if hasSeek:
-                fLoc = filename.tell()
-            else:
-                class _DUMMYFILE:
-                    def __init__(self,filename):
-                        self._filename = filename
-                    def write(self,bytes): pass
-                    def close(self): pass
-                filename = _DUMMYFILE(filename)
+        #better fix for filename is a 'file' problem
+        self._doSave = 0
         passes = 0
         while 1:
-            if passes and isFile and hasSeek: filename.seek(fLoc)
             passes = passes + 1
             if self._onProgress:
                 self.onProgress('PASS', passes)
@@ -662,8 +678,8 @@ class BaseDocTemplate:
             happy = self._allSatisfied()
 
             if happy:
-                if isFile and not hasSeek:
-                    filename = filename._filename
+                self._doSave = 0
+                self.canv.save()
                 break
             if passes > maxPasses:
                 raise IndexError, "Index entries not resolved after %d passes" % maxPasses
